<?php
namespace I18n;

class Makepot extends \MakePOT
{
    protected $domain;

    /** @noinspection PhpMissingParentConstructorInspection
     * @param string $domain
     * @param bool $deprecated
     */
    public function __construct(string $domain, bool $deprecated = true)
    {
        $this->extractor = new StringExtractor($domain, $this->rules);
    }

    protected function get_composer_json()
    {
        $path = getcwd();

        while (true) {
            if (strlen($path) < 4) break;

            $file = $path . '/composer.json';
            if (file_exists($file)) {
                return json_decode(file_get_contents($file), true);
            }

            $path = dirname($path);
        }

        return ['name' => 'Nije composer projekt'];
    }

    protected function get_version()
    {
        $path = getcwd();

        while (true) {
            if (strlen($path) < 4) break;

            $file = $path . '/.git';
            if (file_exists($file) && is_dir($file)) {
                $last = '';
                foreach (scandir($file . '/refs/tags') as $item) {
                    if (preg_match('/^\.+$/', $item)) continue;
                    $last = $item;
                }

                return $last ?: 'Nema releasanu verziju';
            }

            $path = dirname($path);
        }

        return 'Nije git repozitorij';
    }

    protected function _xgettext($project, $dir, $output_file, $placeholders = [], $excludes = [], $includes = [])
    {
        $meta = array_merge($this->meta['default'], $this->meta[$project]);
        $placeholders = array_merge($meta, $placeholders);
        $meta['output'] = $output_file;
        $placeholders['year'] = date('Y');

        $placeholder_keys = array_map(function($x) {
            return sprintf('{%s}', $x);
        }, array_keys($placeholders));

        $placeholder_values = array_values($placeholders);

        foreach($meta as $key => $value) {
            $meta[$key] = str_replace($placeholder_keys, $placeholder_values, $value);
        }

        $originals = $this->extractor->extract_from_directory($dir, $excludes, $includes);
        $composerJson = $this->get_composer_json();

        $pot = new \PO;
        $pot->entries = $originals->entries;

        $pot->set_header( 'Project-Id-Version', $composerJson['name'].' '.$this->get_version());
        $pot->set_header( 'Report-Msgid-Bugs-To', 'web@ep.hr');
        $pot->set_header( 'POT-Creation-Date', gmdate( 'Y-m-d H:i:s+00:00' ) );
        $pot->set_header( 'MIME-Version', '1.0' );
        $pot->set_header( 'Content-Type', 'text/plain; charset=UTF-8' );
        $pot->set_header( 'Content-Transfer-Encoding', '8bit' );
        $pot->set_header( 'PO-Revision-Date', date( 'Y') . '-MO-DA HO:MI+ZONE' );
        $pot->set_header( 'Last-Translator', 'Elektronika Perhoč <web@ep.hr>' );
        $pot->set_header( 'Language-Team', 'Elektronika Perhoč <web@ep.hr>');
        $pot->export_to_file($output_file);

        return true;
    }

    public function generic($dir, $output)
    {
        $this->domain = $output;
        return $this->_xgettext('generic', $dir, $dir . '/' . $output . '.pot', array());
    }
}